/*!
    @file     neopixel_picker.ino
    @author   ladyada, ktown, Becky Stern (Adafruit Industries)

A basic color picker and animation mode toggler using Bluefruit LE & the Bluefruit App!
*/
/**************************************************************************/
#include <string.h>
#include <Arduino.h>
#include <SPI.h>
#include <SoftwareSerial.h>
#include <Adafruit_NeoPixel.h>

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN            6
// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS      72

Adafruit_NeoPixel pixel = Adafruit_NeoPixel(NUMPIXELS, PIN);

#include "Adafruit_BLE.h"
#include "Adafruit_BluefruitLE_SPI.h"
#include "Adafruit_BluefruitLE_UART.h"

// If you are using Hardware Serial
// The following macros declare the Serial port you are using. Uncomment this
// line if you are connecting the BLE to Leonardo/Micro or Flora
#define BLUEFRUIT_HWSERIAL_NAME           Serial1

// Other recommended pins!
#define BLUEFRUIT_UART_MODE_PIN         -1   // Optional but recommended, set to -1 if unused

/*=========================================================================
    APPLICATION SETTINGS

    VERBOSE_MODE            If set to 1 enables full data output (for
                            debugging), otherwise set it to 0 to disable
                            verbose output
    BLE_READPACKET_TIMEOUT  The timeout in ms waiting for a data packet
    -----------------------------------------------------------------------*/
    #define VERBOSE_MODE                    1
    #define BLE_READPACKET_TIMEOUT          500
/*=========================================================================*/

/* ...or hardware serial, which does not need the RTS/CTS pins. Uncomment this line */
Adafruit_BluefruitLE_UART ble(BLUEFRUIT_HWSERIAL_NAME, BLUEFRUIT_UART_MODE_PIN);


// A small helper
void error(const __FlashStringHelper*err) {
  Serial.println(err);
  while (1);
}

// function prototypes over in packetparser.cpp
uint8_t readPacket(Adafruit_BluefruitLE_UART *ble, uint16_t timeout);
float parsefloat(uint8_t *buffer);
void printHex(const uint8_t * data, const uint32_t numBytes);

// the packet buffer
extern uint8_t packetbuffer[];


/**************************************************************************/
/*!
    @brief  Sets up the HW an the BLE module (this function is called
            automatically on startup)
*/
/**************************************************************************/

int R1[] = {25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35};
int R2[] = {24, 23, 22, 21, 20, 19, 18, 17, 16, 15};
int R3[] = {7, 8, 9, 10, 11, 12, 13};
int R4[] = {6, 5, 4, 3, 2, 1, 0};
int L1[] = {46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36};
int L2[] = {47, 48, 49, 50, 51, 52, 53, 54, 55, 56};
int L3[] = {64, 63, 62, 61, 60, 59, 58, 57};
int L4[] = {65, 66, 67, 68, 69, 70, 71};

//Color
    uint8_t red;
    uint8_t green;
    uint8_t blue;
    uint8_t compRed;
    uint8_t compGreen;
    uint8_t compBlue;
    uint8_t triadRed1;
    uint8_t triadRed2;
    uint8_t triadGreen1;
    uint8_t triadGreen2;
    uint8_t triadBlue1;
    uint8_t triadBlue2;
    uint8_t animationState = 1;

void setup(void)
{
  //while (!Serial);
  delay(500);
  
  // turn off neopixel`
  pixel.begin(); // This initializes the NeoPixel library.
  pixel.setBrightness(80); //adjust brightness here
  pixel.setPixelColor(0, pixel.Color(0,0,0)); // off
  pixel.show();
  
  Serial.begin(115200);
  Serial.println(F("Adafruit Bluefruit App Controller Example"));
  Serial.println(F("-----------------------------------------"));

  /* Initialise the module */
  Serial.print(F("Initialising the Bluefruit LE module: "));

    /* Initialise the module */
  Serial.print(F("Initialising the Bluefruit LE module: "));
  if ( !ble.begin(VERBOSE_MODE) )
  {
    error(F("Couldn't find Bluefruit, make sure it's in CoMmanD mode & check wiring?"));
  }
  Serial.println( F("OK!") );

  /* Perform a factory reset to make sure everything is in a known state */
  Serial.print(F("Performing a factory reset: "));
  if (! ble.factoryReset() ){
       error(F("Couldn't factory reset"));
  }
  /* Disable command echo from Bluefruit */
  ble.echo(false);

  Serial.println("Requesting Bluefruit info:");
  /* Print Bluefruit information */
  ble.info();

  Serial.println(F("Please use Adafruit Bluefruit LE app to connect in Controller mode"));
  Serial.println(F("Then activate/use the sensors, color picker, game controller, etc!"));
  Serial.println();
  
  ble.verbose(false);  // debug info is a little annoying after this point!
  
  /* Wait for connection */
  while (! ble.isConnected()) {
      delay(500);
  }
  
  Serial.println(F("*****************"));

  // Set module to DATA mode
  if (BLUEFRUIT_UART_MODE_PIN >= 0) {
    Serial.println( F("Switching to DATA mode using the MODE pin!") );
    ble.setMode(BLUEFRUIT_MODE_DATA);
  } else {
    Serial.println( F("Switching to DATA mode using +++!") );
    ble.println("+++");
    delay(10);
  }

  Serial.println(F("*****************"));

}

/**************************************************************************/
/*!
    @brief  Constantly poll for new command or response data
*/
/**************************************************************************/
void loop(void)
{  
  /* Wait for new data to arrive */
  uint8_t len = readPacket(&ble, BLE_READPACKET_TIMEOUT);
  //if (len == 0) return;
  if (len == 0 && animationState == 1){
    flowingStripes(pixel.Color(red,green,blue), 50);
    flowingStripes(pixel.Color(compRed,compGreen,compBlue), 50);
    //colorWipe(pixel.Color(red,green,blue), 50);
    //colorWipe(pixel.Color(compRed,compGreen,compBlue), 50);
    pixel.show(); // This sends the updated pixel color to the hardware.
  }
    if (len == 0 && animationState == 2){
    for(uint16_t i=0; i<pixel.numPixels(); i++) {
      pixel.setPixelColor(i, pixel.Color(0,0,0));
  }
    flashRandom(5,20);
    pixel.show(); // This sends the updated pixel color to the hardware.
  }
      if (len == 0 && animationState == 3){
    for(uint16_t i=0; i<pixel.numPixels(); i++) {
      pixel.setPixelColor(i, pixel.Color(0,0,0));
  }
    rainbowCycle(20);
  }

  /* Got a packet! */
  // printHex(packetbuffer, len);

  // Color
  if (packetbuffer[1] == 'C') {
    red = packetbuffer[2];
    green = packetbuffer[3];
    blue = packetbuffer[4];
    compRed = 255-red;
    compGreen = 255-green;
    compBlue = 255-blue;
    
    Serial.print ("RGB #"); 
    if (red < 0x10) Serial.print("0");
    Serial.print(red, HEX);
    if (green < 0x10) Serial.print("0");
    Serial.print(green, HEX);
    if (blue < 0x10) Serial.print("0");
    Serial.println(blue, HEX);    
  }
  // Buttons
  if (packetbuffer[1] == 'B') {
    uint8_t buttnum = packetbuffer[2] - '0';
    boolean pressed = packetbuffer[3] - '0';
    Serial.print ("Button "); Serial.print(buttnum);
    animationState = buttnum;
    if (pressed) {
      Serial.println(" pressed");
    } else {
      Serial.println(" released");
    }
  }

}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<pixel.numPixels(); i++) {
      pixel.setPixelColor(i, c);
      pixel.show();
      delay(wait);
  }
}

void flowingStripes(uint32_t c, uint8_t wait) {
for(uint16_t i=0; i<11; i++) {
      pixel.setPixelColor(R1[i], c);
      pixel.setPixelColor(R2[i], c);
      pixel.setPixelColor(R3[i], c);
      pixel.setPixelColor(R4[i], c);
      pixel.setPixelColor(L1[i], c);
      pixel.setPixelColor(L2[i], c);
      pixel.setPixelColor(L3[i], c);
      pixel.setPixelColor(L4[i], c);
      pixel.show();
      delay(30);
  }
}

void flashRandom(int wait, uint8_t howmany) {
 
  for(uint16_t i=0; i<howmany; i++) {
    // get a random pixel from the list
    int j = random(pixel.numPixels());
    
    // now we will 'fade' it in 5 steps
    for (int x=0; x < 5; x++) {
      int r = red * (x+1); r /= 5;
      int g = green * (x+1); g /= 5;
      int b = blue * (x+1); b /= 5;
      
      pixel.setPixelColor(j, pixel.Color(r, g, b));
      pixel.show();
      delay(wait);
    }
    // & fade out in 5 steps
    for (int x=5; x >= 0; x--) {
      int r = red * x; r /= 5;
      int g = green * x; g /= 5;
      int b = blue * x; b /= 5;
      
      pixel.setPixelColor(j, pixel.Color(r, g, b));
      pixel.show();
      delay(wait);
    }
  }
  // LEDs will be off when done (they are faded to 0)
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<pixel.numPixels(); i++) {
      pixel.setPixelColor(i, Wheel((i+j) & 255));
    }
    pixel.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< pixel.numPixels(); i++) {
      pixel.setPixelColor(i, Wheel(((i * 256 / pixel.numPixels()) + j) & 255));
    }
    pixel.show();
    delay(wait);
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  if(WheelPos < 85) {
   return pixel.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  } else if(WheelPos < 170) {
   WheelPos -= 85;
   return pixel.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else {
   WheelPos -= 170;
   return pixel.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
}

